// ReplayDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Sample_MFC.h"
#include "ReplayDlg.h"



// CReplayDlg dialog

IMPLEMENT_DYNAMIC(CReplayDlg, CDialog)
CReplayDlg::CReplayDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CReplayDlg::IDD, pParent)
{
	m_pArtCam		= NULL;
	m_pDoc			= NULL;
	m_ConvertMono	= -1;
	m_hMovieFile	= NULL;
}

CReplayDlg::~CReplayDlg()
{
}

void CReplayDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDS_PLAYTIME, m_PlayTime);
}


BEGIN_MESSAGE_MAP(CReplayDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_LOAD_IMG, OnBnClickedLoadImg)
	ON_BN_CLICKED(IDC_LOAD_MOV, OnBnClickedLoadMov)
	ON_WM_DESTROY()
END_MESSAGE_MAP()


// CReplayDlg message handlers


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Dialog format
//
//
BOOL CReplayDlg::OnInitDialog()
{
	CDialog::OnInitDialog();


	/*****************************************************************************/
	// About this daialog
	// This dialog(CReplayDlg) is created by CSaveRecDlg or CSaveFileDlg
	// on the assumption of that the file is saved by regulated file name
	//
	// eg. xxxx_T.raw  xxxx_D_M.raw 
	//      (_T means temperature data, _M means slide images)
	//
	// Recorded data by Rec function is sequencial data array
	// In order of your purpose, please re-write 
	// process of recording, saving and reading.
	// Please use general application for reading BMP,JPG,PNG,TIFF,AVI.
	/*****************************************************************************/


	// Secure indicating buffer
	BOOL res = CreateImageBuffer();

	// Format view window
	GetDlgItem(IDC_IMAGE)->MoveWindow(10,10,320,240);

	// Slider for play
	m_PlayTime.EnableWindow(FALSE);

	return res;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Finish process
//
//
void CReplayDlg::OnDestroy()
{
	CDialog::OnDestroy();
	CloseMovieFile();
}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Secure range of getting image
//
//
BOOL CReplayDlg::CreateImageBuffer(void)
{
	if(m_pArtCam){
		// Solve memory management in CDib
		if(!m_CDibMono.Create (NULL, m_pArtCam->Width(), m_pArtCam->Height(),  8)) return FALSE;	// BYTE
		if(!m_CDibRaw.Create  (NULL, m_pArtCam->Width(), m_pArtCam->Height(), 16)) return FALSE;	// WORD
		if(!m_CDibColor.Create(NULL, m_pArtCam->Width(), m_pArtCam->Height(), 24)) return FALSE;	// BYTE[3]
		if(!m_CDibTemp.Create (NULL, m_pArtCam->Width(), m_pArtCam->Height(), 32)) return FALSE;	// float
		return TRUE;
	}

	return FALSE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Painting Process
//
//
void CReplayDlg::OnPaint()
{
	CPaintDC dc(this);
	if(-1 == m_ConvertMono)	return;

	// Get and indicate picture controller's HDC
	HWND hWnd = GetDlgItem(IDC_IMAGE)->GetSafeHwnd();
	HDC  hDC  = ::GetDC(hWnd);

	// Switch indication of color / monochrome
	if(0 < m_ConvertMono)
		BitBlt(hDC, 0, 0, m_CDibMono.GetWidth(), m_CDibMono.GetHeight(), m_CDibMono.GetDC(), 0, 0, SRCCOPY);
	else
		BitBlt(hDC, 0, 0, m_CDibColor.GetWidth(), m_CDibColor.GetHeight(), m_CDibColor.GetDC(), 0, 0, SRCCOPY);

	// Certainly release
	::ReleaseDC(hWnd, hDC);
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Process for indicating monochrome
//	
//
void CReplayDlg::ConvertDataToMono(void) 
{
	CString str;
	UINT Size = m_CDibMono.GetWidth() * m_CDibMono.GetHeight();
	UINT i;
	double Scale = 1.0f;

	/**********************************************************************/
	// Received temperatur data distribution is undoubtedly clustered on side.
	// Therefore please get minimum and maximum value to convert to see it as 256 color gradations.
	//*********************************************************************/


	//Convert char, BYTE
	if(1 == m_ConvertMono){
		LPBYTE pSrcImage = m_CDibMono.GetImage();
		BYTE Min = 255, Max = 0;
		for(i=0; i<Size; i++){
			if(Min > pSrcImage[i]) Min = pSrcImage[i];
			if(Max < pSrcImage[i]) Max = pSrcImage[i];
		}

		Scale = 255.0f / ((double)Max - (double)Min);

		for(i=0; i<Size; i++)
			pSrcImage[i] = ((pSrcImage[i] - Min) * Scale ) > 255.0f ? 255 : (BYTE)((pSrcImage[i] - Min) * Scale );

		str.Format(_T("Min:%d   Max:%d"), Min, Max); 
		m_CDibMono.TextOut(0, 0, (LPTSTR)str.GetBuffer());


	//Convert WORD to BYTE
	}else if(2 == m_ConvertMono){
		LPWORD pSrcImage = (LPWORD)m_CDibRaw.GetImage();
		LPBYTE pDstImage = m_CDibMono.GetImage();
		WORD Min = 65535, Max = 0;
		for(i=0; i<Size; i++){
			if(Min > pSrcImage[i]) Min = pSrcImage[i];
			if(Max < pSrcImage[i]) Max = pSrcImage[i];
		}

		Scale = 255.0f / ((double)Max - (double)Min);

		for(i=0; i<Size; i++)
			pDstImage[i] = ((pSrcImage[i] - Min) * Scale ) > 255.0f ? 255 : (BYTE)((pSrcImage[i] - Min) * Scale );

		str.Format(_T("Min:%d   Max:%d"), Min, Max); 
		m_CDibMono.TextOut(0, 0, (LPTSTR)str.GetBuffer());

	//Convert LONG to BYTE
	}else if(3 == m_ConvertMono){
		LPLONG pSrcImage = (LPLONG)m_CDibTemp.GetImage();
		LPBYTE pDstImage = m_CDibMono.GetImage();
		LONG Min = 3000, Max = -3000;
		for(i=0; i<Size; i++){
			if(Min > pSrcImage[i]) Min = pSrcImage[i];
			if(Max < pSrcImage[i]) Max = pSrcImage[i];
		}

		Scale = 255.0f / ((double)Max - (double)Min);

		for(i=0; i<Size; i++)
			pDstImage[i] = ((pSrcImage[i] - Min) * Scale ) > 255.0f ? 255 : (BYTE)((pSrcImage[i] - Min) * Scale );

		str.Format(_T("Min:%d   Max:%d"), Min, Max); 
		m_CDibMono.TextOut(0, 0, (LPTSTR)str.GetBuffer());

	//Convert float to BYTE
	}else if(4 == m_ConvertMono){
		PFLOAT pSrcImage = (PFLOAT)m_CDibTemp.GetImage();
		LPBYTE pDstImage = m_CDibMono.GetImage();
		float Min = 3000.0f, Max = -3000.0f;
		for(i=0; i<Size; i++){
			if(Min > pSrcImage[i]) Min = pSrcImage[i];
			if(Max < pSrcImage[i]) Max = pSrcImage[i];
		}

		Scale = 255.0f / ((double)Max - (double)Min);

		for(i=0; i<Size; i++)
			pDstImage[i] = ((pSrcImage[i] - Min) * Scale ) > 255.0f ? 255 : (BYTE)((pSrcImage[i] - Min) * Scale );

		str.Format(_T("Min:%.1f   Max:%.1f"), Min, Max); 
		m_CDibMono.TextOut(0, 0, (LPTSTR)str.GetBuffer());
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Read a still image
//	
//
void CReplayDlg::OnBnClickedLoadImg()
{
	// Release when slide images are reading
	CloseMovieFile();

	TCHAR szLoadFile[MAX_PATH]=_T("");
	
	// File Dialog
	if(!m_pDoc->GetFileNameDlg(m_hWnd, szLoadFile, _T("RAWfile(*.raw *.dat)\0*.raw;*.dat\0\0"), TRUE, _T("*.raw|*.dat"), _T(""))) {
			return;
	}

	// Get file ID
	HANDLE fp = CreateFile(szLoadFile, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (fp == INVALID_HANDLE_VALUE)
		return;

	DWORD dwResult = 0;

	// Read 3Byte pseudo color data(BYTE[3])
	if(_tcsstr(szLoadFile, _T("_I.raw"))){
		ReadFile(fp, m_CDibColor.GetImage(), m_CDibColor.GetSize(), &dwResult, NULL);
		m_ConvertMono = 0;

	// Read 1Byte temperature data(char, unsigned char)
	} else if(_tcsstr(szLoadFile, _T("_T_C.raw")) || _tcsstr(szLoadFile, _T("_T_B.raw"))){
		ReadFile(fp, m_CDibMono.GetImage(), m_CDibMono.GetSize(), &dwResult, NULL);
		m_ConvertMono = 1;
		ConvertDataToMono();

	// Read 2Byte Raw data(WORD)
	} else if(_tcsstr(szLoadFile, _T("_R.raw"))){
		ReadFile(fp, m_CDibRaw.GetImage(), m_CDibRaw.GetSize(), &dwResult, NULL);
		m_ConvertMono = 2;
		ConvertDataToMono();

	// Read 4Byte temperature data(LONG)
	} else if(_tcsstr(szLoadFile, _T("_T_L.raw"))){
		ReadFile(fp, m_CDibTemp.GetImage(), m_CDibTemp.GetSize(), &dwResult, NULL);
		m_ConvertMono = 3;
		ConvertDataToMono();

	// Read 4Byte temperature data(float)
	} else if(_tcsstr(szLoadFile, _T("_T_F.raw"))){
		ReadFile(fp, m_CDibTemp.GetImage(), m_CDibTemp.GetSize(), &dwResult, NULL);
		m_ConvertMono = 4;
		ConvertDataToMono();
	}
	
	// Close opened file
	CloseHandle(fp);

	// WM_PAINTĂяo
	RECT rc;
	GetClientRect(&rc);
	InvalidateRect(&rc, false);

}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Read slide images
//	
//
void CReplayDlg::OnBnClickedLoadMov()
{
	CloseMovieFile();

	if(NULL == m_pArtCam) 
		return;

	TCHAR szLoadFile[MAX_PATH]= _T("");

	// File Dialog
	if(m_pDoc->GetFileNameDlg(m_hWnd, szLoadFile, _T("RAW file(*.raw *.dat)\0*.raw;*.dat\0\0"), TRUE, _T("*.raw|*.dat"), _T("")))
		_tcscpy(m_szMovieFileName,  szLoadFile);

	// Get file ID
	m_hMovieFile = CreateFile(m_szMovieFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(INVALID_HANDLE_VALUE == m_hMovieFile)
		return CloseMovieFile();

	// Take carefully when you use a file above 2GB
	// It might not work properly especially at an operation systems befor Windows2000
	DWORD	Size = m_pArtCam->Width() * m_pArtCam->Height();
	LONG	FrameCount = 0;
	DWORD	HiWord = 0;
	DWORD64 dwFileEnd = GetFileSize(m_hMovieFile, &HiWord);
	DWORD64 dw64FileSize = (DWORD64)((DWORD64)(HiWord) << 32 | (DWORD64)(dwFileEnd));

	// Calicurate  recorded frame number
	
	// RAW dataF  Frame number = FileSize / Image * (2Byte)
	if(_tcsstr(m_szMovieFileName, _T("_R_M.raw"))){
		FrameCount = LONG(DWORD64(dw64FileSize / (Size * 2)));
	}
	// Temperature dataF Frame number = FileSize / Image size *  (4Byte)
	else if(_tcsstr(m_szMovieFileName, _T("_T_M.raw"))){
		FrameCount = LONG(DWORD64(dw64FileSize / (Size * 4)));
	}


	// If frame number is incorrect
	if (FrameCount == 0)
		return CloseMovieFile();

	// recreate slider by frame number
	m_PlayTime.SetRange(0, FrameCount, TRUE);
	m_PlayTime.SetPos(0);

	// Indicate a first frame and return
	return LoadImageFromMovie(0);
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Read specified 1 frame from slide images
//	
//
void CReplayDlg::LoadImageFromMovie(LONG Pos)
{
	if(NULL == m_pArtCam)
		return;

	// Take carefully when you use a file above 2GB
	// It might not work properly especially at an operation systems befor Windows2000
	DWORD			dwResult = 0;
	LONG			lSize = (m_pArtCam->Width() * m_pArtCam->Height());
	LARGE_INTEGER	Size;


	if(0 <= Pos){
		// Move a file pointer F ImageSize * 2(Byte) * Frame number
		if(_tcsstr(m_szMovieFileName, _T("_R_M.raw"))){
			Size.HighPart = (DWORD)((DWORD64)(lSize * 2 * Pos) >> 32);
			Size.LowPart  = (DWORD)((DWORD64)(lSize * 2 * Pos));
			SetFilePointer(m_hMovieFile, Size.LowPart, &Size.HighPart, FILE_BEGIN);

			// Read 1 frame data from a file
			ReadFile(m_hMovieFile, m_CDibRaw.GetImage(), m_CDibRaw.GetSize(), &dwResult, NULL);
			m_ConvertMono = 2;
			ConvertDataToMono();
		}
		// Move a file pointer F ImageSize * 4(Byte) * Frame number
		else if(_tcsstr(m_szMovieFileName, _T("_T_M.raw"))){
			Size.HighPart = (DWORD)((DWORD64)(lSize * 4 * Pos) >> 32);
			Size.LowPart  = (DWORD)((DWORD64)(lSize * 4 * Pos));
			SetFilePointer(m_hMovieFile, Size.LowPart, &Size.HighPart, FILE_BEGIN);

			// Read 1 frame data from a file
			ReadFile(m_hMovieFile, m_CDibTemp.GetImage(), m_CDibTemp.GetSize(), &dwResult, NULL);
			m_ConvertMono = 4;
			ConvertDataToMono();
		}
	}

	m_PlayTime.EnableWindow(TRUE);

	// Call WM_PAINT
	RECT rc;
	GetClientRect(&rc);
	InvalidateRect(&rc, FALSE);

	return;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Play imageiScrollj
//	
//
void CReplayDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if(m_hMovieFile == NULL)
		return;

	LONG Pos = m_PlayTime.GetPos();
	LoadImageFromMovie(Pos);

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Release slide images file
//	
//
void CReplayDlg::CloseMovieFile(void)
{
	if(m_hMovieFile){
		CloseHandle(m_hMovieFile);
		m_hMovieFile = NULL;
	}

	m_ConvertMono = -1;
	m_PlayTime.EnableWindow(FALSE);
}
