// Sample_MFCView.h : interface of the CSample_MFCView class
//

#pragma once
#include "CArtCamSdk.hpp"
#include "CDib.h"

enum {
	STATUS_ERROR = 0,
	STATUS_FPS,
	STATUS_POINTER,
	STATUS_RAWDATA,
	STATUS_TEMPDATA,
	STATUS_INTTEMP,
};

typedef struct THERMOPARAM{
	LONG	WaitTime;
	LONG	BufferSize;
	LONG	ColorAuto;
	LONG	ColorMin;
	LONG	ColorMax;
	LONG	WaitNuc;
	BOOL	AutoNuc;
	RANGE	Range;
	BOOL	ShowMirrorV;
	BOOL	DataMirrorV;
	LONG	AviMaxSize;
	LONG	RecMode;
	LONG	NowRecording;
}*LPTHERMOPARAM;

#define	DLL_COUNT	14

class CSample_MFCView : public CView
{
protected: // create from serialization only
	CSample_MFCView();
	DECLARE_DYNCREATE(CSample_MFCView)

// Attributes
public:
	CSample_MFCDoc* GetDocument() const;


public:
	/* SDK Class object */
	CArtCamSdk	m_ArtCam;


	/* Option */
	BOOL		m_bShowStop;	// StopEStart frag
	int			m_iDisplayOp;	// Display mode -1:No setting 1:Preview 2:Snapshot
	LONG		m_SelectDevice;	// Connected device(0`7)
	LONG		m_iDeviceCount;	// Connected device number
	LONG		m_iStretchSize;	// Magnification rate
	UINT		m_Timer;		


	/* Graphic */
	CDib	m_CDibMono;			// For indicating monochrome
	CDib	m_CDibTemp;			// For indicating temperature
	CDib	m_CDibRaw;			// For indicating RAW data
	CDib	m_CDibColor;		// For indicating pseudo color

	/* DataEReceived frag */
	struct {
		BOOL	ViewColor;		// TRUE:Color FALSE:Monochrome
		BOOL	GetColor;		// Scan pseudo color data
		BOOL	GetRaw;			// Scan Raw data
		BOOL	GetTemp;		// Scan temperature data
	} m_GetFlg;


	/* Camera setting data */
	THERMOPARAM		m_Param;
	LONG			m_CameraType;
	BOOL			m_DllEnable[DLL_COUNT];
	BOOL			m_Initialized;

protected:
	BOOL	OnDeviceChange		(LONG Number);	// Choose connecting device
	BOOL	CreateImageBuffer	(void);			// Secure buffer for receiving data
	void	StartPreview		(void);			// Start preview
	void	StopPreview			(void);			// Stop preview
	void	ConvertDataToMono	(void);			// Process for indicating monochrome
	void	UpdateStatusBar		(POINT point);	// Update status bar
	void	InitializeArtCam	(LONG Type);	// DLL load, Save default value setting, device check


// Overrides
public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
virtual BOOL PreCreateWindow(CREATESTRUCT& cs);

	struct {
		WORD	Data;
		double	Temperature;
		double	Offset;
	}m_OffsetValue;


// Implementation
public:
	virtual ~CSample_MFCView();
	void	InitMenuPopup(CMenu* pPopupMenu);		// Format menu
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif




// Generated message map functions
protected:
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct); // Start application
	afx_msg void OnDestroy();		// Finish application
	afx_msg void OnPaint();			// Use is instead of OnDraw
	afx_msg void OnShowSnapShot();	// Snapshot
	afx_msg void ShowStop();		// StopEPlay
	afx_msg void FileSave();		// Save 1 frame dialog	
	afx_msg void RecSave();			// Recording dialog
	afx_msg void OpenParamDlg();	// Camera setting dialog
	afx_msg void OnDevice01();		// Device menu 1
	afx_msg void OnDevice02();		// Device menu 2
	afx_msg void OnDevice03();		// Device menu 3
	afx_msg void OnDevice04();		// Device menu 4
	afx_msg void OnDevice05();		// Device menu 5
	afx_msg void OnDevice06();		// Device menu 6
	afx_msg void OnDevice07();		// Device menu 7
	afx_msg void OnDevice08();      // Device menu 8
	afx_msg void OnDisplayX1();		// Magnification x 1 Menu
	afx_msg void OnDisplayX2();		// Magnification x 2 Menu
	afx_msg void OnDisplayX3();		// Magnification x 3 Menu
	afx_msg void OnColorMono();		// Swtch indicating mode
	afx_msg void OnShowPreview();	// Preview
	afx_msg void OnReadTableFile(); // Read temperature table
	afx_msg void OnReadData();		// Read saved data
	afx_msg void OnShowRaw();		// Indicating data to a status bar
	afx_msg void OnShowTemp();		// Indicating data to a status bar
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);		// Process during moving a mouse
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);	// 
	afx_msg LRESULT OnError(WPARAM wParam, LPARAM lParam);		// Call when image is updated
	afx_msg LRESULT OnGraphPaint(WPARAM wParam, LPARAM lParam);	// Call when error is occured
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnSetip();
	afx_msg void OnDll320usb();
	afx_msg void OnDll640usb();
	afx_msg void OnDll320lan();
	afx_msg void OnDll640lan();
	afx_msg void OnDll320gige();
	afx_msg void OnDll640gige();
	afx_msg void OnDll320wom();
	afx_msg void OnDll640wom();
	afx_msg void OnDll320lan2();
	afx_msg void OnDll640lan2();
	afx_msg void OnDll320lan3();
	afx_msg void OnDll640lan3();
	afx_msg void OnDll640p();
	afx_msg void OnDll384p();
	afx_msg void OnTimer(UINT nIDEvent);
};

#ifndef _DEBUG  // debug version in Sample_MFCView.cpp
inline CSample_MFCDoc* CSample_MFCView::GetDocument() const
   { return reinterpret_cast<CSample_MFCDoc*>(m_pDocument); }
#endif

